\name{Krig.sparse}
\alias{Krig.sparse}
\title{
  Kriging surface estimate  
}
\description{Calculating the BLUP or kriging estimate for
  large two dimensional spatial datasets.
}
\usage{Krig.sparse(x,Y,
        cov.fun="expo.cov", cov.fun.args=list(range=10,eps=eps),
        taper.fun="Wu3.cov", taper.fun.args=list(range=10),
        ncov=10000,approxhmax=taper.fun.args$range,		 
        miles=TRUE,R=NULL, eps=10e^-5,
        xM=NULL, m=2,scale.type = "user",x.center = rep(0, ncol(x)),
        x.scale = rep(1, ncol(x)),
        maxiter=25,epsiter=10^-3,
        save.sigma=TRUE,save.chol=FALSE,verbose=FALSE,                        
        nnzmax=100000,tmpmax=10000,...)}
\arguments{
 \item{x}{a \code{m} times \code{2} matrix containing the locations.}
 \item{Y}{the observed values at \code{x}.}

 \item{cov.fun}{Covariance function in the form of an \R function, or its
       	name as a string.}
 \item{cov.fun.args}        cov.fun="expo.cov", cov.fun.args=list(range=10,eps=eps),
        taper.fun="Wu3.cov", taper.fun.args=list(range=10),
        ncov=10000,approxhmax=taper.fun.args$range,		 
        save.sigma=TRUE,save.chol=FALSE,verbose=FALSE,                        
        nnzmax=100000,tmpmax=10000,...){A list with the arguments to call the covariance function
 	(in addition to the locations).}
 \item{taper.fun}{Taper function in the form of an \R function, or its
	name as a string.}
 \item{taper.fun.args}{A list with the arguments to call the taper function
          (in addition to the locations).}
 \item{ncov}{Number of knots to evaluate the approximation}
 \item{approxhmax}{Maximum distance over which the covariance is approximated}

 \item{miles}{logical. If \code{TRUE} (default) distances are in statute
   miles if \code{FALSE} distances in kilometers.}  
 \item{R}{the radius to use for the sphere to find spherical
   distances. If \code{NULL} the radius is either in miles or kilometers
   of the earth depending on the values of the miles argument. If
   \code{R=1} then distances are in radians.}
 \item{eps}{small value, everything smaller is considered zero.}

 \item{xM}{a \code{m} times \code{2} matrix containing the values for 
	the spatial drift. By default, the locations are used.}
 \item{m}{A polynomial function of degree (m-1) will be included in
          the model as the spatial trend (drift) component. }
 \item{scale.type}{A character string among: \code{range}, \code{unit.sd},
          \code{user}, \code{unscaled}. The independent variables are
          scaled to the specified type. See below.}
 \item{x.center}{Centering values to be subtracted from each column of the x
          matrix.}
 \item{x.scale}{Scale values that are divided into each column after
          centering.}
 \item{maxiter}{Maximum number of iterations used in the backfitting 
	iteration}
 \item{epsiter}{Stop the backfitting as soon as the sum of squares of
	the coefficients of two consecutive iterations is smaller.}
 
 \item{save.sigma}{should the covariance matrix be saved (in
   \code{SparseM} format).} 
 \item{save.chol}{should the Cholesky factor of the covariance matrix be
   saved (in \code{SparseM} format).}
 \item{verbose}{should timing and convergence results be printed.}
 \item{nnzmax}{upper bound of non-zero elements in the covariance matrix.}

 \item{tmpmax}{working array for the Cholesky factorisation}
 \item{...}{supplementary parameters that can be given as arguments to
   the function \code{chol}.}
}
\value{\code{Krig.sparse} returns an object of \code{class}
  \code{c("sparse","Krig")}. The second is to reuse many
  handy functions of the library \code{fields}.

An object of the class \code{"sparse"} is a list containing at
least the following components:
 \item{call}{the matched call.}

 \item{iternorm}{norm of coefficients for each iteration.}
 \item{trend}{fitted trend surface at observed values.}
 \item{coef}{coefficients of trend surface.}
 \item{spatial}{spatial part of surface at observed values.}
 \item{solve}{vector used for prediction on other grid points.}
 \item{sigma}{if requested, the covariance matrix.}
 \item{sigmachol}{if requested, the Cholesky factor.}
 \item{timing}{time needed for the main calculations.}
 \item{nnz}{The number of nonzero elements in the covariance matrix and
   its Cholesky factor.}

 Additionally, most input arguments are passed to the object.
}
\details{
  For computational reasons, we do not call simple the \code{solve}
  function but use \code{chol(...)} and \code{backsolve(...)} form the
  \code{SparseM} library.
  We do not allow missing values.
  We only consider two-dimensional domains.

  Concerning the scaling for the spatial trend. By default no 
  scaling is done. Scale type of \code{range} scales the data to 
  the interval (0,1) by forming  \code{(x-min(x))/range(x)} for 
  the x- and y-axis. Scale type of \code{unit.sd} subtracts the 
  mean and divides by the standard deviation. Scale type of 
  \code{user} allows specification of an
  \code{x.center} and \code{x.scale} by the user. The default for \code{user} is
  mean 0 and standard deviation 1. Scale  type of \code{unscaled}
  does not scale the data. 
}
\note{For REALLY big datasets, it would be wise to dissect the
  functions.

  The radius of the earth is assumed to be 3963.34 miles or 6378.388
  kilometers. 

  \code{approxhmax} should be at least as big as the taper range or 
	the domain of the field.

  If \code{nnzmax} is too small, \R may produce a `core dumped'.
}
\seealso{\code{\link{Krig.simple.sparse}},
  \code{\link{predict.sparse}}, \code{\link{plot.sparse}};
  
  \code{transformx} from the  \code{fields} library;
  \code{chol} and \code{backsolve} from the \code{SparseM} library.}
\examples{data(universal)
attach(universal.data)

obj <- Krig.sparse(x, Y,
          cov.fun.args=list(range=10,sill=.9,nugget=.1))

summary(obj)

image.plot( predict.surface(obj))
image.plot( predict.surface(obj, trend.only=TRUE))
}
\keyword{spatial}


